--
-- Definici del procediment SP_I_LOGBD
--
CREATE OR REPLACE PROCEDURE sp_i_LogBd(
		p_procediment IN LOGBD.PROCEDIMENT%TYPE,
		p_param_procediment IN LOGBD.PARAM_PROCEDIMENT%TYPE,
		p_hora_inici IN LOGBD.HORA_INICI%TYPE,
		p_hora_fi IN LOGBD.HORA_FI%TYPE,
		p_resultat IN LOGBD.RESULTAT%TYPE)
IS
BEGIN
	INSERT INTO LOGBD ("ID_LOGBD","PROCEDIMENT","PARAM_PROCEDIMENT","HORA_INICI","HORA_FI","RESULTAT")
	VALUES (s_LogBd.NEXTVAL,p_procediment,p_param_procediment,p_hora_inici,p_hora_fi,p_resultat);
COMMIT;
EXCEPTION
	WHEN OTHERS THEN
	DBMS_OUTPUT.PUT_LINE('Error insert LogBD procediment' || p_procediment);
END;
/

--
-- Definici del procediment SP_S_LOGBD
--
CREATE OR REPLACE PROCEDURE sp_s_Logbd(
	p_data_ini IN Logbd.hora_inici%TYPE,
	p_data_fi IN Logbd.hora_fi%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	v_procediment LOGBD.PROCEDIMENT%TYPE;
	v_param_procediment LOGBD.PARAM_PROCEDIMENT%TYPE;
	v_hora_inici LOGBD.HORA_INICI%TYPE;
	v_hora_fi LOGBD.HORA_FI%TYPE;
	v_resultat LOGBD.RESULTAT%TYPE;
CURSOR r_cursor IS
SELECT * FROM Logbd   WHERE to_date(hora_inici) >= p_data_ini and to_date(hora_inici) <= p_data_fi ORDER BY id_logbd;
BEGIN
SELECT SYSDATE INTO hora_inici FROM DUAL;
FOR r_logbd IN r_cursor LOOP
	DBMS_OUTPUT.PUT_LINE(r_logbd.id_logbd || ' ' || r_logbd.procediment || ' ' || 
	r_logbd.param_procediment || ' ' ||
	r_logbd.hora_inici || ' ' || r_logbd.hora_fi || ' ' || r_logbd.resultat);
END LOOP;
sp_i_logbd('sp_s_Logbd',p_data_ini || ';' || p_data_fi,hora_inici,sysdate,sqlerrm);
EXCEPTION
	WHEN OTHERS THEN
	sp_i_logbd('sp_s_Logbd',p_data_ini || ';' || p_data_fi,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment SP_D_LOGBD
--
CREATE OR REPLACE PROCEDURE sp_d_LogBd( p_id_LogBd IN LogBd.ID_LogBd%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	codi_LogBd LogBd.id_LogBd%TYPE;
BEGIN
SELECT SYSDATE INTO hora_inici FROM DUAL;
SELECT id_LogBd INTO codi_LogBd FROM LogBd WHERE id_LogBd = p_id_LogBd;
DELETE LogBd WHERE ID_LogBd = p_id_LogBd;
DBMS_OUTPUT.PUT_LINE('Registre LogBd esborrat');
sp_i_logbd('sp_d_LogBd',p_id_LogBd,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	DBMS_OUTPUT.PUT_LINE('Registre no trobat');
	sp_i_logbd('sp_d_LogBd',p_id_LogBd,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN 
		ROLLBACK;
	DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_d_LogBd',p_id_LogBd,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definci del procediment sp_inserir_lloc
--
create or replace PROCEDURE sp_inserir_lloc ( p_municipi IN D_Lloc.municipi%TYPE, p_provincia IN D_Lloc.provincia%TYPE,
p_pais IN D_Lloc.pais%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	parametres logbd.param_procediment%TYPE;
	v_id_lloc D_Lloc.id_lloc%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	v_id_lloc:=s_d_lloc.nextval;
	parametres:=v_id_lloc || ';' || p_municipi || ';' || p_provincia || ';' || p_pais;
	INSERT INTO D_LLOC("ID_LLOC", "MUNICIPI", "PROVINCIA", "PAIS")  
  VALUES (v_id_lloc,p_municipi,p_provincia,p_pais);
	DBMS_OUTPUT.PUT_LINE('Un lloc creat');
	sp_i_logbd('sp_inserir_lloc',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN OTHERS THEN
		ROLLBACK;
		DBMS_OUTPUT.PUT_LINE(sqlerrm);
		sp_i_logbd('sp_inserir_lloc',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
--Definici procediment sp_consulta_lloc
--
create or replace PROCEDURE sp_consulta_lloc( p_id_lloc IN D_Lloc.id_lloc%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	id_lloc_lloc D_lloc.id_lloc%TYPE;
  municipi_lloc D_lloc.municipi%TYPE;
  provincia_lloc D_Lloc.provincia%TYPE;
  pais_lloc D_Lloc.pais%TYPE;
  
BEGIN
SELECT SYSDATE INTO hora_inici FROM DUAL;
SELECT id_lloc,municipi,provincia,pais 
INTO id_lloc_lloc,municipi_lloc,provincia_lloc,pais_lloc 
FROM D_Lloc WHERE id_lloc = p_id_lloc;
DBMS_OUTPUT.PUT_LINE('ID:' || p_id_lloc || ' ' || ' Municipi:' || municipi_lloc || ' ' || 'Provncia:' || 
provincia_lloc ||' ' || 'Pas:' || pais_lloc);
sp_i_logbd('sp_consulta_lloc',p_id_lloc,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	DBMS_OUTPUT.PUT_LINE('Registre no trobat');
sp_i_logbd('sp_consulta_lloc',p_id_lloc,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici procediment sp_actualitza_lloc
--
create or replace PROCEDURE sp_actualitza_lloc(
	p_id_lloc IN D_Lloc.id_lloc%TYPE,
  p_municipi IN D_Lloc.municipi%TYPE,
  p_provincia IN D_Lloc.provincia%TYPE,
  p_pais IN D_Lloc.pais%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	rec_lloc D_lloc%ROWTYPE;
	parametres logbd.param_procediment%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	parametres:=p_id_lloc || ';' || p_municipi || ';' || p_provincia || ';' || p_pais;
	SELECT * INTO rec_lloc FROM D_Lloc WHERE id_lloc = p_id_lloc;
	IF p_municipi IS NOT NULL THEN rec_lloc.municipi:=p_municipi;
	END IF;
  IF p_provincia IS NOT NULL THEN rec_lloc.provincia:=p_provincia;
	END IF;
  IF p_pais IS NOT NULL THEN rec_lloc.pais:=p_pais;
	END IF;
	UPDATE D_Lloc SET ROW=rec_lloc WHERE id_lloc=p_id_lloc;
	DBMS_OUTPUT.PUT_LINE('Lloc actualitzat');
	sp_i_logbd('sp_actualitza_lloc',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN 
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_actualitza_lloc',parametres,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
    DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_actualitza_lloc',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_esborra_lloc
--
create or replace PROCEDURE sp_esborra_lloc(p_id_lloc IN D_Lloc.id_lloc%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	v_id_lloc D_lloc.id_lloc%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	SELECT id_lloc INTO v_id_lloc FROM D_Lloc WHERE id_lloc=p_id_lloc;
	DELETE D_Lloc WHERE id_lloc=p_id_lloc;
	DBMS_OUTPUT.PUT_LINE('Registre Lloc esborrat');
	sp_i_logbd('sp_esborra_lloc',p_id_lloc,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_esborra_lloc',p_id_lloc,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
	DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_esborra_lloc',p_id_lloc,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici procediment sp_inserir_temps
--
create or replace PROCEDURE sp_inserir_temps ( p_id_temps IN D_Temps.id_temps%TYPE,
p_dia_mes IN D_Temps.dia_mes%TYPE, p_mes_any IN D_Temps.mes_any%TYPE,
p_any_temps IN D_Temps.any_temps%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	parametres logbd.param_procediment%TYPE;
	
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	
	parametres:=p_id_temps || ';' || p_dia_mes || ';' || p_mes_any || ';' || p_any_temps;
	INSERT INTO D_TEMPS("ID_TEMPS", "DIA_MES", "MES_ANY", "ANY_TEMPS")  
  VALUES (p_id_temps,p_dia_mes,p_mes_any,p_any_temps);
	DBMS_OUTPUT.PUT_LINE('Un temps creat');
	sp_i_logbd('sp_inserir_temps',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN OTHERS THEN
		ROLLBACK;
		DBMS_OUTPUT.PUT_LINE(sqlerrm);
		sp_i_logbd('sp_inserir_temps',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici procediment sp_actualitza_temps
--
create or replace PROCEDURE sp_actualitza_temps(
	 p_id_temps IN D_Temps.id_temps%TYPE,
p_dia_mes IN D_Temps.dia_mes%TYPE, p_mes_any IN D_Temps.mes_any%TYPE,
p_any_temps IN D_Temps.any_temps%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	rec_temps D_temps%ROWTYPE;
	parametres logbd.param_procediment%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	parametres:=p_id_temps || ';' || p_dia_mes || ';' || p_mes_any || ';' || p_any_temps;
	SELECT * INTO rec_temps FROM D_Temps WHERE id_temps = p_id_temps;
	IF p_dia_mes IS NOT NULL THEN rec_temps.dia_mes:=p_dia_mes;
	END IF;
  IF p_mes_any IS NOT NULL THEN rec_temps.mes_any:=p_mes_any;
	END IF;
  IF p_any_temps IS NOT NULL THEN rec_temps.any_temps:=p_any_temps;
	END IF;
	UPDATE D_Temps SET ROW=rec_temps WHERE id_temps=p_id_temps;
	DBMS_OUTPUT.PUT_LINE('Temps actualitzat');
	sp_i_logbd('sp_actualitza_temps',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN 
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_actualitza_temps',parametres,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
    DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_actualitza_temps',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_esborra_temps
--
create or replace PROCEDURE sp_esborra_temps(p_id_temps IN D_Temps.id_temps%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	v_id_temps D_Temps.id_temps%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	SELECT id_temps INTO v_id_temps FROM D_Temps WHERE id_temps=p_id_temps;
	DELETE D_Temps WHERE id_temps=p_id_temps;
	DBMS_OUTPUT.PUT_LINE('Registre Temps esborrat');
	sp_i_logbd('sp_esborra_temps',p_id_temps,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_esborra_temps',p_id_temps,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
	DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_esborra_temps',p_id_temps,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_consulta_temps
--
create or replace PROCEDURE sp_consulta_temps( p_id_temps IN D_temps.id_temps%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	id_temps_temps D_temps.id_temps%TYPE;
  dia_mes_temps D_temps.dia_mes%TYPE;
  mes_any_temps D_Temps.mes_any%TYPE;
  any_temps_temps D_Temps.any_temps%TYPE;
  
  
BEGIN
SELECT SYSDATE INTO hora_inici FROM DUAL;
SELECT id_temps,dia_mes,mes_any,any_temps 
INTO id_temps_temps,dia_mes_temps,mes_any_temps,any_temps_temps 
FROM D_temps WHERE id_temps = p_id_temps;
DBMS_OUTPUT.PUT_LINE('ID:' || p_id_temps || ' ' || ' Dia:' || dia_mes_temps || ' ' || 'Mes:' || 
mes_any_temps ||' ' || 'Any:' || any_temps_temps);
sp_i_logbd('sp_consulta_temps',p_id_temps,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	DBMS_OUTPUT.PUT_LINE('Registre no trobat');
sp_i_logbd('sp_consulta_temps',p_id_temps,hora_inici,sysdate,sqlerrm);
END;
/

--
--Definici del procediment sp_inserir_producte
--
CREATE OR REPLACE PROCEDURE sp_inserir_producte( p_codi_ean IN D_Producte.id_codi_ean%TYPE,p_descripcio IN D_Producte.descripcio%TYPE, p_data IN D_Producte.data_producte%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	parametres logbd.param_procediment%TYPE;
	v_id_producte D_Producte.id_producte%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	v_id_producte:=s_d_producte.nextval;
	parametres:=v_id_producte || ';' || p_codi_ean || ';' || p_descripcio || ';' || p_data;
	INSERT INTO D_PRODUCTE("ID_PRODUCTE", "ID_CODI_EAN", "DESCRIPCIO", "DATA_PRODUCTE")  VALUES (v_id_producte, p_codi_ean, p_descripcio, p_data);
	DBMS_OUTPUT.PUT_LINE('Un producte creat');
	sp_i_logbd('sp_inserir_producte',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN OTHERS THEN
		ROLLBACK;
		DBMS_OUTPUT.PUT_LINE(sqlerrm);
		sp_i_logbd('sp_inserir_producte',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_consulta_producte
--
CREATE OR REPLACE PROCEDURE sp_consulta_producte( p_id_producte IN D_Producte.id_Producte%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	id_producte_producte D_Producte.id_producte%TYPE;
	codi_ean_producte D_Producte.id_codi_ean%TYPE;
	descripcio_producte D_Producte.descripcio%TYPE;
	data_producte D_Producte.data_producte%TYPE;
BEGIN
SELECT SYSDATE INTO hora_inici FROM DUAL;
SELECT id_producte, id_codi_ean, descripcio,data_producte INTO id_producte_producte, codi_ean_producte, descripcio_producte,data_producte FROM D_Producte WHERE id_producte = p_id_producte;
DBMS_OUTPUT.PUT_LINE('ID:' || p_id_producte || ' ' || ' Codi EAN13:' || codi_ean_producte || ' ' || 'Descripci:' || descripcio_producte ||' ' || 'Data entrada catleg:' || data_producte);
sp_i_logbd('sp_consulta_producte',p_id_producte,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	DBMS_OUTPUT.PUT_LINE('Registre no trobat');
sp_i_logbd('sp_consulta_producte',p_id_producte,hora_inici,sysdate,sqlerrm);
END;
/

--
--Definici del procediment SP_esborra_producte
--
CREATE OR REPLACE PROCEDURE sp_esborra_producte(p_id_producte IN D_Producte.id_producte%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	v_id_producte D_Producte.id_producte%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	SELECT id_producte INTO v_id_producte FROM D_Producte WHERE id_producte=p_id_producte;
	DELETE D_Producte WHERE id_producte=p_id_producte;
	DBMS_OUTPUT.PUT_LINE('Registre Producte esborrat');
	sp_i_logbd('sp_esborra_producte',p_id_producte,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_esborra_producte',p_id_producte,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
	DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_esborra_producte',p_id_producte,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_actualitza_producte
--
CREATE OR REPLACE PROCEDURE sp_actualitza_producte(
	p_id_producte IN D_Producte.id_producte%TYPE,
	p_codi_ean IN D_Producte.id_codi_ean%TYPE,
	p_descripcio IN D_Producte.descripcio%TYPE,
	p_data IN D_Producte.data_producte%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	id_producte_producte D_Producte.id_producte%TYPE;
	rec_producte D_Producte%ROWTYPE;
	parametres logbd.param_procediment%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	parametres:=p_id_producte || ';' || p_codi_ean || ';' || p_descripcio || ';' || p_data;
	SELECT * INTO rec_producte FROM D_Producte WHERE id_producte = p_id_producte;
	IF p_codi_ean IS NOT NULL THEN rec_producte.id_codi_ean:=p_codi_ean;
	END IF;
	IF p_descripcio IS NOT NULL THEN rec_producte.descripcio:=p_descripcio;
	END IF;
	IF p_data IS NOT NULL THEN rec_producte.data_producte:=p_data;
	END IF;
	UPDATE D_Producte SET ROW=rec_producte WHERE id_producte=p_id_producte;
	DBMS_OUTPUT.PUT_LINE('Producte actualitzat');
	sp_i_logbd('sp_actualitza_producte',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN 
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_actualitza_producte',parametres,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
	DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_actualitza_producte',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_inserir_botiga
--
create or replace PROCEDURE sp_inserir_botiga (p_id_lloc IN D_Botiga.id_lloc%TYPE,p_mail IN D_Botiga.mail%TYPE, 
p_treballadors IN D_Botiga.treballadors%TYPE, p_franquicia IN D_Botiga.franquicia%TYPE, p_virtual IN D_Botiga.virtual%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	parametres logbd.param_procediment%TYPE;
	v_id_botiga D_Botiga.id_botiga%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	v_id_botiga:=s_d_botiga.nextval;
	parametres:=v_id_botiga || ';' || p_id_lloc || ';' || p_mail || ';' || p_treballadors || ';' || p_franquicia || ';' || p_virtual;
	INSERT INTO D_BOTIGA("ID_BOTIGA", "ID_LLOC", "MAIL", "TREBALLADORS", "FRANQUICIA", "VIRTUAL")  VALUES (v_id_botiga, p_id_lloc, 
  p_mail, p_treballadors,p_franquicia, p_virtual);
	DBMS_OUTPUT.PUT_LINE('Una botiga creada');
	sp_i_logbd('sp_inserir_botiga',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN OTHERS THEN
		ROLLBACK;
		DBMS_OUTPUT.PUT_LINE(sqlerrm);
		sp_i_logbd('sp_inserir_botiga',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_actualitza_botiga
--
create or replace PROCEDURE sp_actualitza_botiga(
	p_id_botiga IN D_Botiga.id_botiga%TYPE,
	p_id_lloc IN D_Botiga.id_lloc%TYPE,
	p_mail IN D_Botiga.mail%TYPE,
	p_treballadors IN D_Botiga.treballadors%TYPE,
  p_franquicia IN D_Botiga.franquicia%TYPE,
  p_virtual IN D_Botiga.virtual%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	id_producte_producte D_Producte.id_producte%TYPE;
	rec_botiga D_Botiga%ROWTYPE;
	parametres logbd.param_procediment%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	parametres:=p_id_botiga || ';' || p_id_lloc || ';' || p_mail || ';' || p_treballadors || ';' || p_franquicia || ';' ||
  p_virtual;
	SELECT * INTO rec_botiga FROM D_Botiga WHERE id_botiga = p_id_botiga;
	IF p_id_lloc IS NOT NULL THEN rec_botiga.id_lloc:=p_id_lloc;
	END IF;
	IF p_mail IS NOT NULL THEN rec_botiga.mail:=p_mail;
	END IF;
	IF p_treballadors IS NOT NULL THEN rec_botiga.treballadors:=p_treballadors;
	END IF;
  IF p_franquicia IS NOT NULL THEN rec_botiga.franquicia:=p_franquicia;
  END IF;
  IF p_virtual IS NOT NULL THEN rec_botiga.virtual:=p_virtual;
  END IF;
	UPDATE D_Botiga SET ROW=rec_botiga WHERE id_botiga=p_id_botiga;
	DBMS_OUTPUT.PUT_LINE('Botiga actualitzada');
	sp_i_logbd('sp_actualitza_botiga',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN 
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_actualitza_botiga',parametres,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
    DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_actualitza_botiga',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_consulta_botiga
--
create or replace PROCEDURE sp_consulta_botiga( p_id_botiga IN D_Botiga.id_Botiga%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	id_botiga_botiga D_Botiga.id_botiga%TYPE;
  id_lloc_botiga D_Botiga.id_lloc%TYPE;
  mail_botiga D_Botiga.mail%TYPE;
  treballadors_botiga D_Botiga.treballadors%TYPE;
  franquicia_botiga D_Botiga.franquicia%TYPE;
  virtual_botiga D_Botiga.virtual%TYPE;
BEGIN
SELECT SYSDATE INTO hora_inici FROM DUAL;
SELECT id_botiga,id_lloc,mail,treballadors,franquicia,virtual 
INTO id_botiga_botiga,id_lloc_botiga,mail_botiga,treballadors_botiga,franquicia_botiga,virtual_botiga
FROM D_Botiga WHERE id_botiga = p_id_botiga;
DBMS_OUTPUT.PUT_LINE('ID:' || p_id_botiga || ' ' || ' Identificador del lloc:' || id_lloc_botiga || ' ' || 'Mail de la botiga:' || 
mail_botiga ||' ' || 'Nmero de treballadors:' || treballadors_botiga || ' ' || 's franqucia?:' || franquicia_botiga ||
 ' ' || 's botiga virtual?:' || virtual_botiga);
sp_i_logbd('sp_consulta_botiga',p_id_botiga,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	DBMS_OUTPUT.PUT_LINE('Registre no trobat');
sp_i_logbd('sp_consulta_botiga',p_id_botiga,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_esborra_botiga
--
create or replace PROCEDURE sp_esborra_botiga(p_id_botiga IN D_Botiga.id_botiga%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	v_id_botiga D_Botiga.id_botiga%TYPE;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	SELECT id_botiga INTO v_id_botiga FROM D_Botiga WHERE id_botiga=p_id_botiga;
	DELETE D_Botiga WHERE id_botiga=p_id_botiga;
	DBMS_OUTPUT.PUT_LINE('Registre Botiga esborrada');
	sp_i_logbd('sp_esborra_botiga',p_id_botiga,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_esborra_botiga',p_id_botiga,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
	DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_esborra_botiga',p_id_botiga,hora_inici,sysdate,sqlerrm);
END;
/



--
--Definici del procediment sp_actualitza_estadistica
--
create or replace PROCEDURE sp_actualitza_estadistica(p_any INTEGER)
IS
	rec_est ESTADISTICA%ROWTYPE;
  hora_inici logbd.hora_inici%TYPE;
	parametres logbd.param_procediment%TYPE;
  percentatge_virtual INTEGER;
  numero INTEGER;
 
BEGIN
percentatge_virtual:=0;

	SELECT SYSDATE INTO hora_inici FROM DUAL;

	SELECT * INTO  rec_est FROM ESTADISTICA WHERE any_estudi=p_any;

--Actualitza benefici cadena
SELECT SUM(ben_net) AS ben_total INTO rec_est.ben_cadena FROM F_Venda INNER JOIN D_Temps ON (F_Venda.id_temps=D_Temps.id_temps)
WHERE D_Temps.any_temps=p_any;

--Actualitza botiga amb ms benefici
SELECT id_botiga,ben_total INTO rec_est.id_botiga,rec_est.benef_botiga FROM ( SELECT id_botiga, SUM(ben_net) AS ben_total 
FROM F_Venda INNER JOIN D_Temps ON (F_Venda.id_temps=D_Temps.id_temps)
WHERE D_Temps.any_temps=p_any GROUP BY id_botiga order by ben_total DESC) WHERE ROWNUM < 2;

--Actualitza producte amb ms unitats venudes
SELECT id_codi_ean,un_total INTO rec_est.id_codi_ean,rec_est.un_prod FROM (SELECT pp.id_codi_ean, SUM(unitats) AS un_total 
FROM D_producte pp INNER JOIN F_Venda v ON (pp.id_producte=v.id_producte) INNER JOIN D_Temps tt ON (v.id_temps=tt.id_temps)
WHERE tt.any_temps=p_any GROUP BY pp.id_codi_ean order by un_total DESC) WHERE ROWNUM < 2;

--Actualitza hora del dia amb ms productes venuts
SELECT hora,num_prod_mes_total INTO rec_est.hora_dia_mes,rec_est.num_prod_mes FROM (SELECT hora, SUM(unitats) AS num_prod_mes_total
        FROM F_Venda INNER JOIN D_Temps ON (F_Venda.id_temps=D_Temps.id_temps)
WHERE D_Temps.any_temps=p_any GROUP BY hora order by num_prod_mes_total DESC) 
    WHERE ROWNUM < 2;

--Actualitza hora del dia amb menys productes venuts
SELECT hora,num_prod_menys_total INTO rec_est.hora_dia_menys,rec_est.num_prod_menys FROM (SELECT hora, SUM(unitats) AS num_prod_menys_total
        FROM F_Venda INNER JOIN D_Temps ON (F_Venda.id_temps=D_Temps.id_temps)
WHERE D_Temps.any_temps=p_any GROUP BY hora order by num_prod_menys_total ASC) 
    WHERE ROWNUM < 2;

--Actualitza dia del mes amb ms productes venuts
SELECT dia_mes,num_dia_mes INTO rec_est.dia_mes,rec_est.num_dia_mes FROM(
SELECT temps.dia_mes, SUM(v.unitats) AS num_dia_mes FROM D_Temps temps INNER JOIN F_Venda v
ON (temps.id_temps=v.id_temps) WHERE temps.any_temps=p_any GROUP BY temps.dia_mes
ORDER BY num_dia_mes DESC) where rownum < 2;

--Actualitza dia del mes amb menys productes venuts
SELECT dia_mes,num_dia_menys INTO rec_est.dia_menys,rec_est.num_dia_menys FROM(
SELECT temps.dia_mes, SUM(v.unitats) AS num_dia_menys FROM D_Temps temps INNER JOIN F_Venda v
ON (temps.id_temps=v.id_temps) WHERE temps.any_temps=p_any GROUP BY temps.dia_mes
ORDER BY num_dia_menys ASC) where rownum < 2;

--Actualitza ciutat amb ms vendes
SELECT municipi,ben_ciutat INTO rec_est.ciutat,rec_est.ben_ciutat FROM(
SELECT lloc.municipi, SUM(v.ben_net) AS ben_ciutat FROM D_Lloc lloc INNER JOIN D_Botiga botiga
ON (lloc.id_lloc=botiga.id_lloc) INNER JOIN F_Venda v ON (botiga.id_botiga=v.id_botiga) 
INNER JOIN D_Temps temps ON(v.id_temps=temps.id_temps)
WHERE temps.any_temps=p_any GROUP BY lloc.municipi ORDER BY ben_ciutat DESC) WHERE ROWNUM < 2;


--Actualitza percentatge 
-- Clcul del benefici de les botigues virtuals. Si no hi ha botigues virtuals posa 0
SELECT COUNT(DISTINCT(v.id_botiga)) INTO numero FROM F_Venda v INNER JOIN D_Temps temps ON (v.id_temps=temps.id_temps)
INNER JOIN D_Botiga botiga ON (v.id_botiga=botiga.id_botiga) WHERE
temps.any_temps=p_any AND botiga.virtual='V';
IF (numero=0) THEN percentatge_virtual:=0;
ELSE
select per_virtual INTO percentatge_virtual FROM (
SELECT sum(v.ben_net) AS per_virtual FROM F_Venda v INNER JOIN D_Botiga botiga
ON (v.id_botiga=botiga.id_botiga) INNER JOIN D_Temps temps ON (v.id_temps=temps.id_temps)
WHERE temps.any_temps=p_any AND botiga.virtual='V' GROUP BY botiga.virtual) ;
END IF;
-- Clcul del percentatge propiament dit
SELECT percentatge INTO rec_est.percentatge FROM(
SELECT percentatge_virtual*100/sum(v.ben_net) AS percentatge FROM F_Venda v 
INNER JOIN D_Temps temps ON (v.id_temps=temps.id_temps) 
WHERE temps.any_temps=p_any);


UPDATE ESTADISTICA SET ROW=rec_est WHERE any_estudi=p_any;

	sp_i_logbd('sp_actualitza_estadistica',p_any,hora_inici,sysdate,sqlerrm);
  
COMMIT;
EXCEPTION
   WHEN NO_DATA_FOUND THEN 
		DBMS_OUTPUT.PUT_LINE('No hi ha cap venda en aquest any');
		sp_i_logbd('sp_actualitza_estadistica',p_any,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
	DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_actualitza_estadistica',p_any,hora_inici,sysdate,sqlerrm);
END;
/


--
-- Definici del procediment sp_inserir_venda
--
create or replace PROCEDURE sp_inserir_venda(
p_id_producte IN F_Venda.id_producte%TYPE,
p_id_botiga IN F_Venda.id_botiga%TYPE,
p_data IN F_Venda.id_temps%TYPE,
p_hora IN F_Venda.hora%TYPE,
p_unitats IN F_Venda.unitats%TYPE,
p_preu IN F_Venda.preu_brut%TYPE,
p_benef IN F_Venda.ben_net%TYPE)
IS
  rec_est ESTADISTICA%ROWTYPE;
  dia INTEGER;
  temps_any INTEGER;
  hora_inici logbd.hora_inici%TYPE;
  parametres logbd.param_procediment%TYPE;
  v_id_venda F_venda.id_venda%TYPE;
  
BEGIN
SELECT SYSDATE INTO hora_inici FROM DUAL;
v_id_venda:=s_f_venda.nextval;
parametres:=v_id_venda || ';' || p_id_producte || ';' || p_id_botiga || ';' || p_data || ';' || p_hora || ';' || p_unitats
|| ';' || p_preu|| ';' || p_benef;
INSERT INTO F_Venda ("ID_VENDA","ID_PRODUCTE","ID_BOTIGA","ID_TEMPS","HORA","UNITATS","PREU_BRUT","BEN_NET") VALUES
(v_id_venda,p_id_producte,p_id_botiga,p_data,p_hora,p_unitats,p_preu,p_benef);
SELECT any_temps INTO temps_any FROM D_temps WHERE id_temps=p_data;

--DBMS_OUTPUT.PUT_LINE(temps_any);
--Al introduir una venda actualitzem la taula Estadistica
sp_actualitza_estadistica(temps_any);

DBMS_OUTPUT.PUT_LINE('Una venda creada');
	sp_i_logbd('sp_inserir_venda',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN OTHERS THEN
		ROLLBACK;
		DBMS_OUTPUT.PUT_LINE(sqlerrm);
		sp_i_logbd('sp_inserir_venda',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici del procediment sp_actaulitza_venda
--
create or replace PROCEDURE sp_actualitza_venda(
	p_id_venda F_Venda.id_venda%TYPE,
	p_id_producte F_Venda.id_producte%TYPE,
  p_id_botiga F_Venda.id_botiga%TYPE,
  p_data_venda F_Venda.id_temps%TYPE,
  p_hora F_Venda.hora%TYPE,
  p_unitats F_Venda.unitats%TYPE,
  p_preu_brut F_Venda.preu_brut%TYPE,
  p_ben_net F_Venda.ben_net%TYPE)

IS
	hora_inici logbd.hora_inici%TYPE;
	id_venda_venda F_Venda.id_venda%TYPE;
	rec_venda F_Venda%ROWTYPE;
	parametres logbd.param_procediment%TYPE;
  temps_any INTEGER;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	parametres:=p_id_venda || ';' || p_id_producte || ';' || p_id_botiga || ';' || p_data_venda || ';' ||p_hora || ';' ||
  p_unitats || ';' ||p_preu_brut || ';' ||p_ben_net;
	SELECT * INTO rec_venda FROM F_Venda WHERE id_venda = p_id_venda;
	IF p_id_producte IS NOT NULL THEN rec_venda.id_producte:=p_id_producte;
	END IF;
	IF p_id_botiga IS NOT NULL THEN rec_venda.id_botiga:=p_id_botiga;
	END IF;
	IF p_data_venda IS NOT NULL THEN rec_venda.id_temps:=p_data_venda;
	END IF;
  IF p_hora IS NOT NULL THEN rec_venda.hora:=p_hora;
	END IF;
  IF p_unitats IS NOT NULL THEN rec_venda.unitats:=p_unitats;
	END IF;
  IF p_preu_brut IS NOT NULL THEN rec_venda.preu_brut:=p_preu_brut;
	END IF;
  IF p_ben_net IS NOT NULL THEN rec_venda.ben_net:=p_ben_net;
	END IF;
  
	UPDATE F_Venda SET ROW=rec_venda WHERE id_venda=p_id_venda;
	DBMS_OUTPUT.PUT_LINE('Venda_actualitzada');
  SELECT any_temps INTO temps_any FROM D_temps WHERE id_temps=p_data_venda;
  --sp_actualitza_estadistica(to_number(to_char(p_data_venda,'YYYY'),'9999'));
	sp_i_logbd('sp_actualitza_venda',parametres,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN 
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_actualitza_venda',parametres,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
	DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_actualitza_venda',parametres,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici procediment sp_esborra_venda
--
create or replace PROCEDURE sp_esborra_venda(p_id_venda IN F_Venda.id_venda%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	v_id_venda F_Venda.id_venda%TYPE;
  p_any INTEGER;
BEGIN
	SELECT SYSDATE INTO hora_inici FROM DUAL;
	SELECT id_venda INTO v_id_venda FROM F_Venda WHERE id_venda=p_id_venda;
  SELECT any_temps INTO p_any FROM D_temps INNER JOIN F_Venda ON (d_temps.id_temps=F_venda.id_temps) 
  WHERE F_Venda.id_venda=p_id_venda;
  sp_actualitza_estadistica(p_any);
	DELETE F_Venda WHERE id_venda=p_id_venda;
	DBMS_OUTPUT.PUT_LINE('Registre Venda esborrada');
	sp_i_logbd('sp_esborra_venda',p_id_venda,hora_inici,sysdate,sqlerrm);
  
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
		DBMS_OUTPUT.PUT_LINE('Registre no trobat');
		sp_i_logbd('sp_esborra_venda',p_id_venda,hora_inici,sysdate,sqlerrm);
	WHEN OTHERS THEN
		ROLLBACK;
	DBMS_OUTPUT.PUT_LINE(sqlerrm);
	sp_i_logbd('sp_esborra_venda',p_id_venda,hora_inici,sysdate,sqlerrm);
END;
/

--
-- Definici procediment sp_consulta_venda
--
create or replace PROCEDURE sp_consulta_venda( p_id_venda IN F_Venda.id_venda%TYPE)
IS
	hora_inici logbd.hora_inici%TYPE;
	id_venda_venda F_Venda.id_venda%TYPE;
	id_producte_venda F_Venda.id_producte%TYPE;
  id_botiga_venda F_Venda.id_botiga%TYPE;
  data_venda_venda F_Venda.id_temps%TYPE;
  hora_venda F_Venda.hora%TYPE;
  unitats_venda F_Venda.unitats%TYPE;
  preu_brut_venda F_Venda.preu_brut%TYPE;
  ben_net_venda F_Venda.ben_net%TYPE;
BEGIN
SELECT SYSDATE INTO hora_inici FROM DUAL;
SELECT id_venda,id_producte,id_botiga,id_temps,hora,unitats,preu_brut,ben_net 
INTO id_venda_venda,id_producte_venda,id_botiga_venda,data_venda_venda,hora_venda,unitats_venda,preu_brut_venda,ben_net_venda
FROM F_Venda WHERE id_venda = p_id_venda;
DBMS_OUTPUT.PUT_LINE('ID:' || p_id_venda || ' ' || ' Codi producte:' || id_producte_venda || ' ' || 
'Codi Botiga:' || id_botiga_venda ||' ' || 'Data venda:' || data_venda_venda || ' ' || 'Hora venda:' || hora_venda || ' ' ||
'Unitats venudes:' || unitats_venda || ' ' || 'Preu brut:' || preu_brut_venda || ' ' ||'Benefici net:' || ben_net_venda);
sp_i_logbd('sp_consulta_venda',p_id_venda,hora_inici,sysdate,sqlerrm);
COMMIT;
EXCEPTION
	WHEN NO_DATA_FOUND THEN
	DBMS_OUTPUT.PUT_LINE('Registre no trobat');
sp_i_logbd('sp_consulta_venda',p_id_venda,hora_inici,sysdate,sqlerrm);
END;
/






 

