--
--Definici de la taula D_Lloc
--
CREATE TABLE D_LLOC(
	ID_LLOC NUMBER NOT NULL,
	MUNICIPI VARCHAR2(40 BYTE) NOT NULL,
	PROVINCIA VARCHAR2(25 BYTE) NOT NULL,
	PAIS VARCHAR2(25 BYTE) NOT NULL,
	CONSTRAINT "D_LLOC_PK" PRIMARY KEY(ID_LLOC))
TABLESPACE USERS
STORAGE(
	INITIAL 64K
	MAXEXTENTS UNLIMITED
)
LOGGING;

CREATE UNIQUE INDEX D_LLOC_INDEX1 ON D_LLOC (MUNICIPI, PROVINCIA, PAIS)
TABLESPACE USERS
STORAGE(
	INITIAL 64K
	MAXEXTENTS UNLIMITED
)
LOGGING;

--
-- Definici de la taula D_TEMPS
--
CREATE TABLE D_TEMPS(
	ID_TEMPS NUMBER NOT NULL,
	DIA_MES NUMBER NOT NULL,
	MES_ANY NUMBER NOT NULL,
	ANY_TEMPS NUMBER NOT NULL,	
	CONSTRAINT "D_TEMPS_PK" PRIMARY KEY(ID_TEMPS))
TABLESPACE USERS
STORAGE(
	INITIAL 64K
	MAXEXTENTS UNLIMITED
)
LOGGING;


--
-- Definici de la taula D_Botiga
--
CREATE TABLE D_BOTIGA(
id_botiga NUMBER NOT NULL,
id_lloc NUMBER NOT NULL,
mail VARCHAR2(100 BYTE) NOT NULL,
treballadors NUMBER NOT NULL,
franquicia CHAR(1) NOT NULL,
virtual CHAR(1) NOT NULL,
CONSTRAINT "D_Botiga PK" PRIMARY KEY(id_botiga),
CONSTRAINT FK_D_Lloc_D_Botiga FOREIGN KEY (id_lloc) REFERENCES D_Lloc(id_lloc) ON DELETE CASCADE,
CONSTRAINT CK_franquicia CHECK (franquicia IN ('S','N')),
CONSTRAINT CK_virtual CHECK (virtual IN ('V','F')))
TABLESPACE USERS STORAGE(
INITIAL 64K
MAXEXTENTS UNLIMITED
)
LOGGING;

--
-- Definici de la taula D_PRODUCTE
--
CREATE TABLE D_PRODUCTE(
id_producte NUMBER NOT NULL,
id_codi_ean NUMBER NOT NULL,
descripcio VARCHAR2(255 BYTE) NOT NULL,
data_producte DATE  NOT NULL,
CONSTRAINT "D_Producte PK" PRIMARY KEY(id_producte))
TABLESPACE USERS STORAGE(
INITIAL 64K
MAXEXTENTS UNLIMITED
)
LOGGING;

--
-- Definici de la taula LOGBD
--
CREATE TABLE LOGBD(
	id_logbd NUMBER NOT NULL,
	procediment VARCHAR2(50 BYTE) NOT NULL,
	param_procediment VARCHAR2(255 BYTE),
	hora_inici DATE NOT NULL,
	hora_fi DATE NOT NULL,
	resultat VARCHAR2(255 BYTE) NOT NULL,
	CONSTRAINT "LOGBD PK" PRIMARY KEY(id_logbd))
TABLESPACE USERS
STORAGE(
	INITIAL 64K
	MAXEXTENTS UNLIMITED
)
LOGGING;


--
-- Definici de la taula F_VENDA
--
CREATE TABLE F_VENDA(
	id_venda NUMBER NOT NULL,
	id_producte NUMBER NOT NULL,
	id_botiga NUMBER NOT NULL,
	id_temps NUMBER NOT NULL,
	hora NUMBER NOT NULL,
	unitats NUMBER NOT NULL,
	preu_brut NUMBER NOT NULL,
	ben_net NUMBER NOT NULL,
	CONSTRAINT "F_VENDA PK" PRIMARY KEY(id_venda),
	CONSTRAINT FK_D_PRODUCTE_F_VENDA FOREIGN KEY(id_producte) REFERENCES D_PRODUCTE(id_producte) ON DELETE CASCADE,
	CONSTRAINT FK_D_BOTIGA_F_VENDA FOREIGN KEY(id_botiga) REFERENCES D_BOTIGA(id_botiga) ON DELETE CASCADE,
	CONSTRAINT FK_D_TEMPS_F_VENDA FOREIGN KEY(id_temps) REFERENCES D_TEMPS(id_temps) ON DELETE CASCADE)
TABLESPACE USERS
STORAGE(
	INITIAL 64K
	MAXEXTENTS UNLIMITED
)
LOGGING;

--
--Definici taula estadstica
--
CREATE TABLE  ESTADISTICA(
  id_estadistica NUMBER NOT NULL,
  any_estudi NUMBER NOT NULL,
  ben_cadena NUMBER NULL,
  id_botiga NUMBER NULL,
  benef_botiga NUMBER NULL,
  id_codi_ean NUMBER NULL,
  un_prod NUMBER NULL,
  hora_dia_mes NUMBER NULL,
  num_prod_mes NUMBER NULL,
  hora_dia_menys NUMBER NULL,
  num_prod_menys NUMBER NULL,
  dia_mes NUMBER NULL,
  num_dia_mes NUMBER NULL,
  dia_menys NUMBER NULL,
  num_dia_menys NUMBER NULL,
  ciutat VARCHAR2(50) NULL,
  ben_ciutat NUMBER NULL,
  percentatge NUMBER NULL,
CONSTRAINT CK_BEN_CADENA CHECK (BEN_CADENA>=0),
CONSTRAINT CK_BENEF_BOTIGA CHECK (BENEF_BOTIGA>=0),
CONSTRAINT CK_UN_PROD CHECK (UN_PROD>=0),
CONSTRAINT CK_HORA_DIA_MES CHECK (HORA_DIA_MES>=0 AND HORA_DIA_MES<=24),
CONSTRAINT CK_NUM_PROD_MES CHECK(NUM_PROD_MES>=0),
CONSTRAINT CK_HORA_DIA_MENYS CHECK (HORA_DIA_MENYS>=0 AND HORA_DIA_MENYS<=24),
CONSTRAINT CK_NUM_PROD_MENYS CHECK (NUM_PROD_MENYS>=0),
CONSTRAINT CK_DIA_MES CHECK (DIA_MES>=0 AND DIA_MES<=31),
CONSTRAINT CK_DIA_MENYS CHECK (DIA_MENYS>=0 AND DIA_MENYS<=31),
CONSTRAINT CK_NUM_DIA_MES CHECK (NUM_DIA_MES>=0),
CONSTRAINT CK_NUM_DIA_MENYS CHECK (NUM_DIA_MENYS>=0),
CONSTRAINT CK_BEN_CIUTAT CHECK (BEN_CIUTAT>=0),
CONSTRAINT CK_PERCENTATGE CHECK (PERCENTATGE>=0 AND PERCENTATGE<=100),
CONSTRAINT "ESTADISTICA PK" PRIMARY KEY(id_estadistica))
TABLESPACE USERS
STORAGE(
	INITIAL 64K
	MAXEXTENTS UNLIMITED
)
LOGGING;

